#include "InterruptedDownloads.h"
#include "InterruptedFile.h"
#include "QtellaSub.h"
#include "Logging.h"
#include "StringManipulation.h"
#include "SearchDialog.h"
#include "InterruptedFileSearch.h"
#include "Icon.h"
#include "DownloadManager.h"

#include <qlistview.h>
#include <qlistview.h>
#include <qdir.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>

InterruptedDownloads::InterruptedDownloads(QtellaSub* parent, std::string directory)
  : _parent(parent)
{
  if(!directory.empty())
    if(directory[directory.size()-1] != '/') directory += "/";
  _directory = directory;
  
  _context_menu = new QPopupMenu(parent, "Interrupted Downloads");
  _context_menu->insertItem( "Search for file", this, SLOT(searchSelectedFiles()), CTRL+Key_C);
  _context_menu->insertItem( "Resume", this, SLOT(resumeSelectedFiles()), CTRL+Key_S);
  _context_menu->insertSeparator();
  _context_menu->insertItem( "Abort Download", this, SLOT(abortSelectedFiles()), CTRL+Key_A);
  _context_menu->insertItem( "Delete from disc", this, SLOT(deleteSelectedFiles()), CTRL+Key_Delete);
  _context_menu->insertSeparator();
  _context_menu->insertItem( "Launch", parent, SLOT(slotLaunchInterruptedFile()), CTRL+Key_L);
  

  update();
}

void InterruptedDownloads::update()
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  while(pos != _file_map.end())
    {
      delete pos->first;
      ++pos;
    }
  _file_map.clear();

  QDir d(_directory.c_str(), "*", QDir::Name|QDir::IgnoreCase, QDir::Files);

  for(int i = 0; i < d.count(); ++i)
    {
      InterruptedFile ifile(_directory + "/" + std::string(d[i]));
      
      if(!ifile.isValid()) continue;

      QListViewItem* item = new QListViewItem(_parent->ui_listview_interrupt);
	  
      item->setText(0, ifile.filename().c_str());
      item->setText(1, ifile.ext().c_str());
      item->setText(2, ifile.strsize().c_str());
      item->setText(3, ifile.strprogress().c_str());
      item->setText(4, ifile.ip().c_str());

      _file_map.insert( make_pair(item, ifile) );
    }
}

void InterruptedDownloads::getSelectedFiles(std::vector<InterruptedFile>& files)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->first->isSelected()) files.push_back(pos->second);
}

void InterruptedDownloads::deleteSelectedFiles()
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); )
    if(pos->first->isSelected())
      {
	if( pos->second.getColored() )
	  {
	    std::map<QListViewItem*, QueryHitEntry>::iterator p = _qhe.find(pos->first);
	    
	    if(p != _qhe.end())
	      QtellaSub::getInstance()->download_manager->deleteDownload(p->second);
	  }
	
	if(!QFile::remove(pos->second.completeName().c_str()))
	  Logging::getInstance()->log(Logging::Error, "Could not remove file [" + pos->second.completeName() + "]");
	delete pos->first;
	_file_map.erase(pos++);
      }
    else ++pos;
}

void InterruptedDownloads::searchSelectedFiles()
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  
  while(pos != _file_map.end())
    {
      if( !pos->first->isSelected() ) { ++pos; continue; }

      std::vector<std::string> v;
      StringManipulation sm(pos->second.filename());
      sm.extractWords(v);

      std::vector<std::string> search_words;
      for(int i = 0; i < v.size(); ++i)
	if(search_words.size() < 3) search_words.push_back(v[i]);
	else
	  {
	    int j = 0;
	    for(int k = 0; k < search_words.size(); ++k)
	      if(search_words[k].size() < search_words[j].size()) j = k;

	    if(search_words[j].size() < v[i].size()) search_words[j] = v[i];
	  }

      std::string search;
      for(int i = 0; i < search_words.size(); ++i) search += search_words[i] + " ";
      if(!search.empty()) search = search.substr(0, search.size()-1);

      SearchDialog d(0, 0, true);
      d.setQuery(search);
      if(d.exec() == SearchDialog::Accepted) _parent->search(d.getQuery());

      ++pos;
    }
}

void InterruptedDownloads::resumeSelectedFiles()
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  
  while(pos != _file_map.end())
    {
      if( !pos->first->isSelected() ) { ++pos; continue; }

      if( pos->second.getColored() )
	{
	  QMessageBox::information(0, "Information", "An interrupted file can be resumed only once.", QMessageBox::Ok);
	  ++pos;
	  continue;
	}

      InterruptedFileSearch d(0, 0, true);
      d.init(pos->second.size());
      if(d.exec() == InterruptedFileSearch::Accepted)
	{
	  QueryHitEntry q = *d.getSelectedQueryHit();

	  QListViewItem* item = q.item;
	  q.item = NULL; // do not destroy item when contructor is called

	  _qhe.insert( make_pair(pos->first, q) );
	  _parent->download_manager->addDownload(q, item, pos->second.filename(), pos->first);
	}

      ++pos;
    }
}

void InterruptedDownloads::newQueryHit(unsigned long fsize)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->second.size() == fsize) 
      {
	pos->second.incFound();
	updateBall(pos->first);
      }
}

void InterruptedDownloads::removeQueryHit(unsigned long fsize)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->second.size() == fsize) 
      {
	pos->second.decFound();
	updateBall(pos->first);
      }
}

void InterruptedDownloads::setYellow(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getYellowBall()));
}

void InterruptedDownloads::setRed(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getRedBall()));
}

void InterruptedDownloads::setGreen(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getGreenBall()));
}

void InterruptedDownloads::setSmile(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getSmile()));
}

void InterruptedDownloads::setGreenSmile(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getGreenSmile()));
}

void InterruptedDownloads::updateBall(QListViewItem *item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;

  if( pos->second.getFound() > 0 )
    if( pos->second.getColored() == 0 ) item->setPixmap(0, *(Icon::getInstance()->getGreyBall()));

  if( pos->second.getFound() == 0)
    if( pos->second.getColored() == 0) item->setPixmap(0, QPixmap());
}

void InterruptedDownloads::clearColored(QListViewItem* item)
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.clearColored();
  updateBall(item);
}

void InterruptedDownloads::slotShowContextMenu(QListViewItem* item, const QPoint& p, int i)
{
  _context_menu->popup(p);
}

void InterruptedDownloads::abortSelectedFiles()
{
  std::map<QListViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if( (pos->first->isSelected()) && (pos->second.getColored()) )
      {
	std::map<QListViewItem*, QueryHitEntry>::iterator p = _qhe.find(pos->first);
	
	if(p != _qhe.end())
	  QtellaSub::getInstance()->download_manager->abortDownload(p->second);
      }
}
